import os
from media_transporter import config
from . import Logger, MediaFile


class TvFile(MediaFile):
    """Class to handle specific actions when dealing with a TV show file.

    TvFile performs tv-specific tasks, including specific logging
    messages and checking for already-existing shows/episodes on the media share.

    Attributes
    ----------
        download_path: str
            Full path to the file being operated on.
        file_name: str
            Name of the file being operated on.
        show_info: tuple
            Tuple consisting of matched regular expression groups identifying
            the specific file as a TV show.

    """

    def __init__(self, download_path, file_name, show_info):
        title, episode_id, season, episode = show_info
        self.episode_id = episode_id
        self.season = int(season)
        self.episode = int(episode)
        MediaFile.__init__(self, download_path, file_name, title)

    def prepare_destination(self):
        """Checks if there is storage capacity on the media share,
        and if there is, attempt to make the TV show's directory structure
        on the media share in advance of extracting and moving."""
        self.tv_root_path = config.share_tv_root_path % (
            self.share_path, self.title)
        self.tv_season_path = config.share_tv_season_path % (
            self.share_path, self.title, self.season)

        if os.path.isdir(self.tv_root_path):
            if self.capacity_reached():
                Logger.log('[!] Capacity reached. Skipping adding Season %s of %s.' % (
                    self.season, self.title))
            else:
                if not os.path.isdir(self.tv_season_path):
                    Logger.log('[+] Creating folder for Season %s of %s' %
                               (self.season, self.title))
                    os.mkdir(self.tv_season_path)
        else:
            if self.capacity_reached():
                Logger.log(
                    '[!] Capacity reached. Skipping adding show %s.' % self.title)
            else:
                Logger.log('[+] Adding TV Show: %s' % self.title)
                os.makedirs(self.tv_season_path)

    def process(self):
        """Checks if a TV show already has a directory created on the media share.
        Directly moves a media file to its destination, or extracts and moves the
        movie from a RAR archive."""
        try:
            existing_episodes = os.listdir(self.tv_season_path)
        except:
            os.makedirs(self.tv_season_path)
            existing_episodes = os.listdir(self.tv_season_path)

        if not [episode for episode in existing_episodes if self.episode_id in episode]:
            if self.has_video_extension:
                self.move_media()
            else:
                self.extract_media()



safe_capacity_percentage = 100
media_shares = "/Users/steve/Desktop/Media"

logfile = "transmission.log"
download_path = "~/Downloads"
unrar_path = "/usr/local/bin/unrar"

regex_movie = "^(.+).(\d{4}).+(240p|480p|720p|1080p|2160p).+$"
regex_tv = "^(.+).(S(\d+)E(\d+)).+$"
share_tv_root_path = "%s/TV/%s"
share_tv_season_path = "%s/TV/%s/Season %s"
share_movie_root_path = "%s/Movies/%s"



import os
import subprocess
from . import TransportException
from media_transporter import config


class Storage():
    """Class to manage storage-specific functions related to
    dealing with a configured media share.

    Attributes
    ----------
        share_path: str
            First available media share path with available capacity.

    """

    def __init__(self):
        self.share_path = self.locate_media_share()

    def locate_media_share(self):
        """Reads the `media_shares` config string and chooses
        the first available media share path.

        Returns
        -------
            string
                Path to media share.
            TransportException
                Raised exception when media share was not found.
        """
        media_shares = config.media_shares
        for path in media_shares.split(','):
            if os.path.isdir(path):
                return path
            raise TransportException(
                '[!] Media Share %s was not found, terminating...' % path, True)

    def get_volume_capacity(self):
        """Determines the storage capacity of the targeted media share.

        TODO: customize for use with other, non-UNIX systems

        Returns
        -------
            list
                System output of available disk space is on the selected
                media share
        """
        command = """df -H %s | tail -n1 | awk '{ print $5 " " $4 }'""" % self.share_path
        command_output = subprocess.check_output(command, shell=True)
        return command_output.split(' ')

    def capacity_reached(self):
        """Interprets the result of `get_volume_capacity()` and checks
        if it is above or below the `safe_capacity_percentage` config value.

        Returns
        -------
            bool
                True when below the capacity, False when at or above the capacity.
        """

        percentage_free, available_space = self.get_volume_capacity()
        percentage_free = int(percentage_free.replace('%', ''))
        if percentage_free >= int(config.safe_capacity_percentage):
            return True
        return False



import sys
from . import Logger


class TransportException(Exception):
    """Class to record and optionally exit script execution.

    Extends Exception. Uses `Logger` class to write errors to disk.

    Attributes
    ----------
        message: str
            Message to be logged.
        exit: bool
            Determines if the `TransportException` should also
            exit script execution (for fatal errors only).

    """

    def __init__(self, message, exit=False):
        super(TransportException, self).__init__(message)
        Logger.log(message)
        if exit:
            sys.exit(1)



import os
from . import MediaFile, Logger
from media_transporter import config


class MovieFile(MediaFile):
    """Class to handle specific actions when dealing with a Movie file.

    MovieFile performs movie-specific tasks, including specific logging
    messages and checking for already-existing Movies on the media share.

    Attributes
    ----------
        download_path: str
            Full path to the file being operated on.
        file_name: str
            Name of the file being operated on.
        movie_info: tuple
            Tuple consisting of matched regular expression groups identifying
            the specific file as a Movie.

    """

    def __init__(self, download_path, file_name, movie_info):
        title, year, resolution = movie_info
        MediaFile.__init__(self, download_path, file_name, title)

    def prepare_destination(self):
        """Checks if there is storage capacity on the media share,
        and if there is, attempt to make the Movie's directory structure
        on the media share in advance of extracting and moving."""
        self.movie_root_path = config.share_movie_root_path % (
            self.share_path, self.title)

        if os.path.isdir(self.movie_root_path):
            if self.capacity_reached():
                Logger.log(
                    '[!] Capacity reached. Skipping adding movie %s.' % self.title)
            else:
                if not os.path.isdir(self.movie_root_path):
                    Logger.log('[+] Adding Movie: %s' % self.title)
                    os.mkdir(self.movie_root_path)

    def process(self):
        """Checks if a Movie already has a directory created on the media share.
        Directly moves a media file to its destination, or extracts and moves the
        movie from a RAR archive."""
        try:
            existing_movie = os.listdir(self.movie_root_path)
        except:
            os.makedirs(self.movie_root_path)
            existing_movie = os.listdir(self.movie_root_path)

        if not [movie for movie in existing_movie if self.title in movie]:
            if self.has_video_extension:
                self.move_media()
            else:
                self.extract_media()



import os
from datetime import datetime
from media_transporter import config

script_path = '%s/../' % os.path.dirname(os.path.abspath(__file__))
"""string: path to script being executed."""


class Logger():
    """Responsible for logging messages from the script and writing to disk."""
    @staticmethod
    def log(text):
        """Static method to open a log file and write messages generated by
        the script.

        Parameters
        ----------
        text
            Message to log

        Returns
        -------
        None
        """
        dt = datetime.now()
        with open('%s/%s' % (script_path, config.logfile), 'a') as fh:
            fh.write('[%s] %s\n' % (dt.strftime('%Y-%m-%d %H:%M:%S'), text))
        print '[%s] %s' % (dt.strftime('%Y-%m-%d %H:%M:%S'), text)



from distutils.core import setup

setup(
    name='media-transporter',
    version='0.1.3',
    author='Steve Coward',
    author_email='steve.coward@gmail.com',
    url='https://github.com/stevecoward/transporter',
    license='LICENSE.txt',
    description='Manages the transportation of TV/Movie files to a mountable media share.',
    long_description=open('README.txt').read(),
    scripts=['bin/media-transporter'],
    packages=['media_transporter', 'media_transporter.classes'],
)



import os
import shutil
import re
import glob
import subprocess
from . import Storage, Logger, TransportException
from media_transporter import config

flatten_list = lambda l: [item for sublist in l for item in sublist]
"""lambda: lambda function to flatten a list of nested lists to a single list."""


class MediaFile(Storage):
    """Generic class to perform tasks required by both TvFile and MovieFile.

    MediaFile mainly handles common functions such as extracting and moving
    media files from one place to another. This class is also responsible
    for light text formatting of media titles.

    MediaFile inherits from the Storage class, allowing MediaFile to access
    methods and parameters essential to the process of moving files from one
    place to another.

    Attributes
    ----------
        download_path: str
            Full path to the file being operated on.
        file_name: str
            Name of file being operated on.
        title: string
            Original name of the media file. `title` is
            transformed to a human-readable format inline.
        has_video_extension: bool
            Returns True when `file_name` has an acceptable
            media file extension.

    """

    def __init__(self, download_path, file_name, title):
        self.file_name = file_name
        self.title = re.sub(r'(\.|_){1,}', ' ', title)
        self.download_path = download_path
        self.has_video_extension = self.file_name.endswith(
            ('mkv', 'avi', 'mp4', 'mov'))

        if os.path.isdir('%s/%s' % (download_path, file_name)):
            os.chdir('%s/%s' % (download_path, file_name))
        Storage.__init__(self)

    def move_media(self):
        """Moves a file from a download path to its final destination
        on the media share."""
        from . import TvFile, MovieFile

        if isinstance(self, MovieFile):
            Logger.log('[-] Movie %s wasn\'t found. Moving...' % self.title)
        elif isinstance(self, TvFile):
            Logger.log('[-] %s Season %s Episode %s wasn\'t found. Moving...' %
                       self.title, self.season, self.episode)
        shutil.move('%s/%s' % (self.download_path, self.file_name),
                    self.movie_root_path)

    def extract_media(self):
        """Extracts a media file from a series of RAR files, then moves it
        to its final destination on the media share."""
        from . import TvFile, MovieFile

        unrar_command = ''
        rar_files = glob.glob('*.rar')
        if rar_files:
            unrar_command = '%s x %s &>/dev/null' % (
                config.unrar_path, rar_files[0])

        try:
            destination_path = ''
            message = '[-] %s wasn\'t found. Extracting and moving...'
            if isinstance(self, TvFile):
                destination_path = self.tv_season_path
                Logger.log(message % '%s Season %s Episode %s' %
                           (self.title, self.season, self.episode))
            elif isinstance(self, MovieFile):
                destination_path = self.movie_root_path
                Logger.log(message % self.title)

            if rar_files:
                subprocess.check_output(unrar_command, shell=True)
            extracted_files = flatten_list([glob.glob(extension) for extension in [
                                           '*.mkv', '*.avi', '*.mp4', '*.mov']])
            for file in extracted_files:
                shutil.move(file, destination_path)
        except Exception as e:
            raise TransportException('[!] runtime error: %s.' % e, True)
